__version__ = "0.0.1"

from pynicehash.miningdevice import MiningDevice
from pynicehash.miningrig import MiningRig
from pynicehash.nicehash import NiceHash
from pynicehash.mining_status import MiningStatus